==> klayout-0.17.2.tar.gz: klayout-0.17.2/src/dbOASISReader.h <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/



#ifndef HDR_dbOASISReader
#define HDR_dbOASISReader

#include "tlException.h"
#include "tlInternational.h"
#include "tlProgress.h"
#include "tlString.h"

#include "dbLayout.h"
#include "dbReader.h"
#include "dbOASIS.h"
#include "tlStream.h"
#include "dbStreamLayers.h"
#include "dbPropertiesRepository.h"

#include <map>
#include <set>

namespace db
{

/**
 *  @brief Generic base class of OASIS reader exceptions
 */
class OASISReaderException
  : public ReaderException
{
public:
  OASISReaderException (const std::string &msg, size_t p, const std::string &cell)
    : ReaderException (tl::sprintf (tl::translate ("%s (position=%ld, cell=%s)").c_str (), msg.c_str (), p, cell.c_str ()))
  { }
};

/**
 *  @brief The OASIS format stream reader
 */
class OASISReader
  : public ReaderBase, 
    public OASISDiagnostics
{
public: 
  typedef std::vector<tl::Variant> property_value_list;

  /**
   *  @brief Construct a stream reader object
   *
   *  @param s The stream delegate from which to read stream data from
   */
  OASISReader (tl::InputStream &s);

  /**  
   *  @brief Destructor
   */
  ~OASISReader ();

  /** 
   *  @brief The basic read method 
   *
   *  This method will read the stream data and translate this to
   *  insert calls into the layout object. This will not do much
   *  on the layout object beside inserting the objects.
   *  It can be passed a layer map which tells which
   *  OASIS layer(s) to read on which logical layers.
   *  In addition, a flag can be passed that tells wheter to create 
   *  new layers. The returned map will contain all layers, the passed
   *  ones and the newly created ones.
   *
   *  @param layout The layout object to write to
   *  @param map The LayerMap object
   *  @param create true, if new layers should be created
   *  @return The LayerMap object that tells where which layer was loaded
   */
  virtual const LayerMap &read (db::Layout &layout, const LayerMap &map, bool create) throw (tl::Exception);

  /** 
   *  @brief The basic read method (without mapping)
   *
   *  This method will read the stream data and translate this to
   *  insert calls into the layout object. This will not do much
   *  on the layout object beside inserting the objects.
   *  This version will read all input layers and return a map
   *  which tells which OASIS layer has been read into which logical
   *  layer.
   *
   *  @param layout The layout object to write to
   *  @return The LayerMap object
   */
  virtual const LayerMap &read (db::Layout &layout) throw (tl::Exception);

  /**
   *  @brief Accessor method to the database unit
   */
  virtual double dbu () const { return m_dbu; }

  /**
   *  @brief Format
   */
  virtual const char *format () const { return "OASIS"; }

  /**
   *  @brief Issue an error with positional informations
   *
   *  Reimplements OASISDiagnostics
   */
  virtual void error (const std::string &txt) throw (tl::Exception);

  /**
   *  @brief Issue a warning with positional informations
   *
   *  Reimplements OASISDiagnostics
   */
  virtual void warn (const std::string &txt);

private:
  tl::InputStream &m_stream;
  LayerMap m_layer_map;
  tl::Progress m_progress;
  std::string m_cellname;
  double m_dbu;

  modal_variable<Repetition> mm_repetition;
  modal_variable<db::cell_index_type> mm_placement_cell;
  modal_variable<db::Coord> mm_placement_x;
  modal_variable<db::Coord> mm_placement_y;
  modal_variable<unsigned int> mm_layer;
  modal_variable<unsigned int> mm_datatype;
  modal_variable<unsigned int> mm_textlayer;
  modal_variable<unsigned int> mm_texttype;
  modal_variable<db::Coord> mm_text_x;
  modal_variable<db::Coord> mm_text_y;
  modal_variable<std::string> mm_text_string;
  modal_variable<db::Coord> mm_geometry_x;
  modal_variable<db::Coord> mm_geometry_y;
  modal_variable<db::Coord> mm_geometry_w;
  modal_variable<db::Coord> mm_geometry_h;
  modal_variable< std::vector<db::Point> > mm_polygon_point_list;
  modal_variable<db::Coord> mm_path_halfwidth;
  modal_variable<db::Coord> mm_path_start_extension;
  modal_variable<db::Coord> mm_path_end_extension;
  modal_variable< std::vector<db::Point> > mm_path_point_list;
  modal_variable<unsigned int> mm_ctrapezoid_type;
  modal_variable<db::Coord> mm_circle_radius;
  modal_variable<db::PropertiesRepository::property_names_id_type> mm_last_property_name;
  modal_variable<bool> mm_last_property_is_sprop;
  modal_variable<property_value_list> mm_last_value_list;

  std::map <unsigned long, std::string> m_cellnames;
  std::map <unsigned long, std::string> m_textstrings;
  std::map <unsigned long, std::string> m_propstrings;
  std::map <unsigned long, std::string> m_propnames;
  tl::interval_map <db::ld_type, tl::interval_map <db::ld_type, std::string> > m_layernames;

  std::map <unsigned long, db::cell_index_type> m_cells_by_id;
  std::map <unsigned long, db::cell_index_type> m_forward_references;
  std::map <std::string, db::cell_index_type> m_cells_by_name;
  bool m_create_layers;

  std::set <unsigned long> m_defined_cells_by_id;
  std::set <std::string> m_defined_cells_by_name;

  std::map <unsigned long, db::PropertiesRepository::property_names_id_type> m_propname_forward_references;
  std::set <unsigned long> m_propvalue_forward_references;
  db::PropertiesRepository::property_names_id_type m_s_gds_property_name_id;

  void do_read (db::Layout &layout) throw (tl::Exception);
  void do_read_cell (db::cell_index_type cell_index, 
                     db::Layout &layout,
                     tl::vector<db::CellInstArray> &instances,
                     tl::vector<db::CellInstArrayWithProperties> &instances_with_props) throw (tl::Exception);

  void do_read_placement (unsigned char r,
                          bool xy_absolute,
                          db::Layout &layout, 
                          tl::vector<db::CellInstArray> &instances,
                          tl::vector<db::CellInstArrayWithProperties> &instances_with_props) throw (tl::Exception);

  void do_read_text (bool xy_absolute,db::cell_index_type cell_index, db::Layout &layout) throw (tl::Exception);
  void do_read_rectangle (bool xy_absolute,db::cell_index_type cell_index, db::Layout &layout) throw (tl::Exception);
  void do_read_polygon (bool xy_absolute,db::cell_index_type cell_index, db::Layout &layout) throw (tl::Exception);
  void do_read_path (bool xy_absolute,db::cell_index_type cell_index, db::Layout &layout) throw (tl::Exception);
  void do_read_trapezoid (unsigned char r, bool xy_absolute,db::cell_index_type cell_index, db::Layout &layout) throw (tl::Exception);
  void do_read_ctrapezoid (bool xy_absolute,db::cell_index_type cell_index, db::Layout &layout) throw (tl::Exception);
  void do_read_circle (bool xy_absolute,db::cell_index_type cell_index, db::Layout &layout) throw (tl::Exception);

  void read_repetition ();
  void read_pointlist (modal_variable <std::vector <db::Point> > &pointlist, bool for_polygon);
  void read_properties (db::PropertiesRepository &rep);
  std::pair <bool, db::PropertiesRepository::properties_id_type> read_element_properties (db::PropertiesRepository &rep);

  unsigned char get_byte () throw (tl::Exception)
  {
    unsigned char *b = (unsigned char *) m_stream.get (1);
    if (! b) {
      error (tl::translate ("Unexpected end-of-file"));
    }
    return *b;
  }

  long get_long () throw (tl::Exception);
  unsigned long get_ulong () throw (tl::Exception);
  int get_int () throw (tl::Exception);
  unsigned int get_uint () throw (tl::Exception);

  void get (long &l) throw (tl::Exception)
  {
    l = get_long ();
  }

  void get (unsigned long &l) throw (tl::Exception)
  {
    l = get_ulong ();
  }

  void get (int &l) throw (tl::Exception)
  {
    l = get_int ();
  }

  void get (unsigned int &l) throw (tl::Exception)
  {
    l = get_uint ();
  }

  void get (double &d) throw (tl::Exception)
  {
    d = get_real ();
  }

  std::string get_str () throw (tl::Exception);
  void get_str (std::string &s) throw (tl::Exception);
  double get_real () throw (tl::Exception);
  db::Point get_gdelta (long grid = 1) throw (tl::Exception);
  db::Point get_3delta (long grid = 1) throw (tl::Exception);
  db::Point get_2delta (long grid = 1) throw (tl::Exception);
  db::Coord get_coord (long grid = 1) throw (tl::Exception);
  db::Coord get_ucoord (unsigned long grid = 1) throw (tl::Exception);

  std::pair <bool, unsigned int> open_dl (db::Layout &layout, const LDPair &dl, bool create) throw (tl::Exception);
};

}

#endif

==> klayout-0.19.3.tar.gz: klayout-0.19.3/src/dbOASISReader.h <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/



#ifndef HDR_dbOASISReader
#define HDR_dbOASISReader

#include "tlException.h"
#include "tlInternational.h"
#include "tlProgress.h"
#include "tlString.h"

#include "dbLayout.h"
#include "dbReader.h"
#include "dbOASIS.h"
#include "tlStream.h"
#include "dbStreamLayers.h"
#include "dbPropertiesRepository.h"

#include <map>
#include <set>

namespace db
{

/**
 *  @brief Generic base class of OASIS reader exceptions
 */
class OASISReaderException
  : public ReaderException
{
public:
  OASISReaderException (const std::string &msg, size_t p, const std::string &cell)
    : ReaderException (tl::sprintf (tl::translate ("%s (position=%ld, cell=%s)").c_str (), msg.c_str (), p, cell.c_str ()))
  { }
};

/**
 *  @brief The OASIS format stream reader
 */
class OASISReader
  : public ReaderBase, 
    public OASISDiagnostics
{
public: 
  typedef std::vector<tl::Variant> property_value_list;

  /**
   *  @brief Construct a stream reader object
   *
   *  @param s The stream delegate from which to read stream data from
   */
  OASISReader (tl::InputStream &s);

  /**  
   *  @brief Destructor
   */
  ~OASISReader ();

  /** 
   *  @brief The basic read method 
   *
   *  This method will read the stream data and translate this to
   *  insert calls into the layout object. This will not do much
   *  on the layout object beside inserting the objects.
   *  A set of options can be specified with the LoadLayoutOptions
   *  object.
   *  The returned map will contain all layers, the passed
   *  ones and the newly created ones.
   *
   *  @param layout The layout object to write to
   *  @param map The LayerMap object
   *  @param create true, if new layers should be created
   *  @return The LayerMap object that tells where which layer was loaded
   */
  virtual const LayerMap &read (db::Layout &layout, const LoadLayoutOptions &options) throw (tl::Exception);

  /** 
   *  @brief The basic read method (without mapping)
   *
   *  This method will read the stream data and translate this to
   *  insert calls into the layout object. This will not do much
   *  on the layout object beside inserting the objects.
   *  This version will read all input layers and return a map
   *  which tells which OASIS layer has been read into which logical
   *  layer.
   *
   *  @param layout The layout object to write to
   *  @return The LayerMap object
   */
  virtual const LayerMap &read (db::Layout &layout) throw (tl::Exception);

  /**
   *  @brief Format
   */
  virtual const char *format () const { return "OASIS"; }

  /**
   *  @brief Issue an error with positional informations
   *
   *  Reimplements OASISDiagnostics
   */
  virtual void error (const std::string &txt) throw (tl::Exception);

  /**
   *  @brief Issue a warning with positional informations
   *
   *  Reimplements OASISDiagnostics
   */
  virtual void warn (const std::string &txt);

private:
  tl::InputStream &m_stream;
  LayerMap m_layer_map;
  tl::Progress m_progress;
  std::string m_cellname;
  double m_dbu;

  modal_variable<Repetition> mm_repetition;
  modal_variable<db::cell_index_type> mm_placement_cell;
  modal_variable<db::Coord> mm_placement_x;
  modal_variable<db::Coord> mm_placement_y;
  modal_variable<unsigned int> mm_layer;
  modal_variable<unsigned int> mm_datatype;
  modal_variable<unsigned int> mm_textlayer;
  modal_variable<unsigned int> mm_texttype;
  modal_variable<db::Coord> mm_text_x;
  modal_variable<db::Coord> mm_text_y;
  modal_variable<std::string> mm_text_string;
  modal_variable<unsigned int> mm_text_string_id;
  modal_variable<db::Coord> mm_geometry_x;
  modal_variable<db::Coord> mm_geometry_y;
  modal_variable<db::Coord> mm_geometry_w;
  modal_variable<db::Coord> mm_geometry_h;
  modal_variable< std::vector<db::Point> > mm_polygon_point_list;
  modal_variable<db::Coord> mm_path_halfwidth;
  modal_variable<db::Coord> mm_path_start_extension;
  modal_variable<db::Coord> mm_path_end_extension;
  modal_variable< std::vector<db::Point> > mm_path_point_list;
  modal_variable<unsigned int> mm_ctrapezoid_type;
  modal_variable<db::Coord> mm_circle_radius;
  modal_variable<db::PropertiesRepository::property_names_id_type> mm_last_property_name;
  modal_variable<bool> mm_last_property_is_sprop;
  modal_variable<property_value_list> mm_last_value_list;

  std::map <unsigned long, std::string> m_cellnames;
  std::map <unsigned long, std::string> m_textstrings;
  std::map <unsigned long, const db::StringRef *> m_text_forward_references;
  std::map <unsigned long, std::string> m_propstrings;
  std::map <unsigned long, std::string> m_propnames;
  tl::interval_map <db::ld_type, tl::interval_map <db::ld_type, std::string> > m_layernames;

  std::map <unsigned long, db::cell_index_type> m_cells_by_id;
  std::map <unsigned long, db::cell_index_type> m_forward_references;
  std::map <std::string, db::cell_index_type> m_cells_by_name;
  bool m_create_layers;
  bool m_read_texts;
  bool m_read_properties;

  std::set <unsigned long> m_defined_cells_by_id;
  std::set <std::string> m_defined_cells_by_name;

  std::map <unsigned long, db::PropertiesRepository::property_names_id_type> m_propname_forward_references;
  std::set <unsigned long> m_propvalue_forward_references;
  db::PropertiesRepository::property_names_id_type m_s_gds_property_name_id;

  void do_read (db::Layout &layout) throw (tl::Exception);
  void do_read_cell (db::cell_index_type cell_index, 
                     db::Layout &layout,
                     tl::vector<db::CellInstArray> &instances,
                     tl::vector<db::CellInstArrayWithProperties> &instances_with_props) throw (tl::Exception);

  void do_read_placement (unsigned char r,
                          bool xy_absolute,
                          db::Layout &layout, 
                          tl::vector<db::CellInstArray> &instances,
                          tl::vector<db::CellInstArrayWithProperties> &instances_with_props) throw (tl::Exception);

  void do_read_text (bool xy_absolute,db::cell_index_type cell_index, db::Layout &layout) throw (tl::Exception);
  void do_read_rectangle (bool xy_absolute,db::cell_index_type cell_index, db::Layout &layout) throw (tl::Exception);
  void do_read_polygon (bool xy_absolute,db::cell_index_type cell_index, db::Layout &layout) throw (tl::Exception);
  void do_read_path (bool xy_absolute,db::cell_index_type cell_index, db::Layout &layout) throw (tl::Exception);
  void do_read_trapezoid (unsigned char r, bool xy_absolute,db::cell_index_type cell_index, db::Layout &layout) throw (tl::Exception);
  void do_read_ctrapezoid (bool xy_absolute,db::cell_index_type cell_index, db::Layout &layout) throw (tl::Exception);
  void do_read_circle (bool xy_absolute,db::cell_index_type cell_index, db::Layout &layout) throw (tl::Exception);

  void read_repetition ();
  void read_pointlist (modal_variable <std::vector <db::Point> > &pointlist, bool for_polygon);
  void read_properties (db::PropertiesRepository &rep);
  std::pair <bool, db::PropertiesRepository::properties_id_type> read_element_properties (db::PropertiesRepository &rep);

  unsigned char get_byte () throw (tl::Exception)
  {
    unsigned char *b = (unsigned char *) m_stream.get (1);
    if (! b) {
      error (tl::translate ("Unexpected end-of-file"));
    }
    return *b;
  }

  long get_long () throw (tl::Exception);
  unsigned long get_ulong () throw (tl::Exception);
  int get_int () throw (tl::Exception);
  unsigned int get_uint () throw (tl::Exception);

  void get (long &l) throw (tl::Exception)
  {
    l = get_long ();
  }

  void get (unsigned long &l) throw (tl::Exception)
  {
    l = get_ulong ();
  }

  void get (int &l) throw (tl::Exception)
  {
    l = get_int ();
  }

  void get (unsigned int &l) throw (tl::Exception)
  {
    l = get_uint ();
  }

  void get (double &d) throw (tl::Exception)
  {
    d = get_real ();
  }

  std::string get_str () throw (tl::Exception);
  void get_str (std::string &s) throw (tl::Exception);
  double get_real () throw (tl::Exception);
  db::Point get_gdelta (long grid = 1) throw (tl::Exception);
  db::Point get_3delta (long grid = 1) throw (tl::Exception);
  db::Point get_2delta (long grid = 1) throw (tl::Exception);
  db::Coord get_coord (long grid = 1) throw (tl::Exception);
  db::Coord get_ucoord (unsigned long grid = 1) throw (tl::Exception);

  std::pair <bool, unsigned int> open_dl (db::Layout &layout, const LDPair &dl, bool create) throw (tl::Exception);
};

}

#endif
